<?php
namespace Teknisa\EST\Modules\Ajuste\Controllers;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\DataSet;
use Teknisa\EST\Modules\Ajuste\Services\AjusteEstoque;
use Teknisa\EST\Services\ValidacoesEstoqueService;
use Teknisa\Libs\Util\Utilities;

class MobileAjusteEstoque {

    protected $AjusteService;
    protected $ValidacoesEstController;
    protected $connection;
    protected $entityManager;
    protected $utilities;

    public function __construct (AjusteEstoque $AjusteEstoque, ValidacoesEstoqueService $ValidacoesEstoqueService, $entityManager, Utilities $utilities) {
        $this->AjusteService = $AjusteEstoque;
        $this->validacoesEstoqueService = $ValidacoesEstoqueService;
        $this->connection = $entityManager->getConnection();
        $this->entityManager = $entityManager;
        $this->utilities = $utilities;
    }

    public function saveInServer ($request, $response) {
        $adjustments = $request->getDataSet()->getRows();

        try {
            if(count($adjustments) > 0){
                $rowAjustments = $adjustments[0];
                $filialLanc = $rowAjustments['CDFILIAL'];
                $this->utilities->saveCollectedData($filialLanc,'Lançamento APP Inventário','2443');
            }
            $this->connection->beginTransaction();
            foreach ($adjustments as &$adjustment) {
                
                //cria campo para salvar erros no ajuste
                $adjustment['ERROR'] = null;
                $adjustment['ERRORTYPE'] = null;
                
                //Valida parametros de datas do estoque
                $mensagemErro = $this->validaDataLancamento($adjustment);
                if($mensagemErro){
                    $adjustment['ERROR'] = $mensagemErro;
                    $adjustment['ERRORTYPE'] = 1; //1 - erros de data (ajuste não é salvo no banco) 2 - erro de atualização ajuste é salvo no banco mas nao atualiza estoque.
                }
                
                if(is_null($adjustment['ERROR'])){
                    //Verifica se o ajuste contém produtos
                    $this->AjusteService->containsProducts($adjustment['PRODUCTS']);
                    //Verifica se os protudos do ajustes são realmente daquel ajuste, comparando a pk que os liga (NRLANCESTQ)
                    $this->AjusteService->checkRelationship($adjustment['NRLANCESTQ'], $adjustment['PRODUCTS']);
                    //Gera um sequencial valido para o ajuste
                    $adjustment['NRLANCESTQ'] = $this->AjusteService->generatedValidSequential();
                    //Seta os campos padrões do ajuste
                    $adjustment = $this->AjusteService->setDefaultColumnsForAdjustment($adjustment);
                    //Substitui o sequencial dos produtos pelo sequencial valido do ajuste
                    $adjustment['PRODUCTS'] = $this->AjusteService->setValidSequential($adjustment['NRLANCESTQ'], $adjustment['PRODUCTS']);
                    //Verifica se todos os produtos tiveram os sequencial trocado, ainda tendo relação com o ajuste
                    $this->AjusteService->checkRelationship($adjustment['NRLANCESTQ'], $adjustment['PRODUCTS']);
                    //Seta os campos padrões do ajuste
                    $adjustment['PRODUCTS'] = $this->AjusteService->setDefaultColumnsForProducts($adjustment['PRODUCTS'], $adjustment['DTLANCESTQ'], $adjustment['CDFILIMOVI']);
                    //Consolida produtos de compra em produtos de estoque
                    $appProducts = $adjustment['PRODUCTS'];//guarda lancamentos originais do app antes de consolidar
                    $adjustment['PRODUCTS'] = $this->AjusteService->consolidadeToStockProducts($adjustment['PRODUCTS']);
                    //Salva o ajuste
                    $this->AjusteService->saveAdjustment($adjustment);
                    $this->entityManager->flush();
                    
                    
                    //Salva os produtos do ajuste
                    foreach ($adjustment['PRODUCTS'] as $product) {
                        $this->AjusteService->saveProducts($product);
                        $this->entityManager->flush();
                    }
                    //restaura os produtos originais não consolidados lançados no app para retornar o datasource.
                    $adjustment['PRODUCTS'] = $appProducts; //restaura lancamento original do app para devolver na requisição
                    $adjustment = $this->AjusteService->changeIsNew($adjustment);
                }
                
                
                
            }
            //$this->connection->rollback();
            $this->connection->commit();
            
            //caso cdfilimovi não esteja setado significa que houve erro de validação de data do primeiro lançamento
            //porem isto nao deve impedir da rotina proseguir para o restante dos lançamentos.
            if(!isset($adjustments[0]['CDFILIMOVI']))
                $adjustments[0]['CDFILIMOVI'] = $adjustments[0]['CDFILIAL'];
                
            $boolAppAtuEstq = $this->AjusteService->getParamAppAtuEstq($adjustments[0]['CDFILIMOVI']);
            
            //Rotina para atualizar o estoque automticamente caso parametro seja true
            if($boolAppAtuEstq){
                foreach ($adjustments as &$adjustment) {
                    if(is_null($adjustment['ERROR'])){
                        if($adjustment['UPDATEMODE'] == 'total')
                            $mensagemErro = $this->zeraInventario($adjustment);
                        else
                            $mensagemErro = $this->atualizaEstoque($adjustment);
                            
                        if($mensagemErro){
                            $adjustment['ERROR'] = $mensagemErro;
                            $adjustment['ERRORTYPE'] = 2; //1 - erros de data (ajuste não é salvo no banco) 2 - erro de atualização ajuste é salvo no banco mas nao atualiza estoque.
                        }
                    }
                }
            }
            
            
            $response->setDataSets(array(new DataSet('adjustments', $adjustments)));
        } catch (\Exception $e) {
            $this->connection->rollback();
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function validaDataLancamento($row) {
        try {
            $dadosValidacao = $row;
            

            $filial = $dadosValidacao['CDFILIAL'];
            $dtLancamento = $dadosValidacao['DTLANCESTQ'];
            $operacao = 'A'; //ajuste || implantacao
            $sistema = 'EST';
            $dtEmissao = $dadosValidacao['DTLANCESTQ'];

            $this->validacoesEstoqueService->validaDataLancamento(
                $filial, $dtLancamento, $operacao, $sistema, $dtEmissao
            );
        } catch (\Exception $e) {
            return $e->getMessage();
        }
    }
    
    public function zeraInventario($row) {
        try {
            //$this->connection->beginTransaction();
            $filial = $row['CDFILIAL'];
            $lancamento = $row['NRLANCESTQ'];
            $idordem = 'U';

            $this->validacoesEstoqueService->zeraInventario($filial, $lancamento, $idordem);
            
            //$this->connection->commit();
        } catch(\Doctrine\DBAL\DBALException $e) {
            return $e->getMessage();
            //$this->connection->rollback();
        } catch (\Exception $e) {
            return $e->getMessage();
            //$this->connection->rollback();
            //$response->setError(new Error($e->getmessage(), $e->getCode()));
        }
    }
    
    public function atualizaEstoque($row) {
        try {
            $payload = $row;

            #gets relevant data from payload
            $cdfilial              = $payload['CDFILIAL'];
            $nrlancestq            = $payload['NRLANCESTQ'];
            $verifyIrregularities  = true; //verificar este parametro
            $abortAtIrregularities = true; //verificar este parametro
            $seqItemCodes          = null;

            #calls procedure validation, preparation and execution, receiving any irregular products, if applicable
            $irregularProducts = $this->validacoesEstoqueService->atualizaEstoque(
              
              
                $cdfilial, $nrlancestq, $verifyIrregularities, $abortAtIrregularities, $seqItemCodes
            );

            #sends data to frontend through response's dataset
            $response->addDataset(new DataSet("irregularProducts", $irregularProducts));
        } catch(\Doctrine\DBAL\DBALException $e) {
            #catches doctrine exceptions and prepares message for response's error
            return $e->getMessage();

            $response->setError(new Error($message, $e->getCode()));
        } catch (\Exception $e) {
            #catches generic error and sends message to frontend through response's error
            return $e->getMessage();
        }
    }
        
    public function getMaxNrSeqItemAjuste(DTO\Request\Row $request, DTO\Response $response) {
        $row = $request->getRow();
    
        $nrsequitem = $this->AjusteService->getMaxNrSeqItemAjuste($row["CDFILIAL"], $row["NRLANCESTQ"]);
        
        $response->addDataset(new DataSet("getMaxNrSeqItemAjuste", array($nrsequitem)));
    }
}